/***************************************************************************
Copyright (c) 2006 Peter Korzuszek, Damian Rakowski, KateOS Project
 
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
 
*Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.
 
*Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE. 
****************************************************************************/

#ifndef SMART_TOOLS_ARGS_H
#define SMART_TOOLS_ARGS_H

#include "smarttools.h"
#include "ststring.h"
#include "stvaluelist.h"
#include "ststringlist.h"

class StArgs
{
	typedef struct{
		StStringList names;
		StStringList values;
		SmartTools::OptionValues valueNum;
		bool allowMultipleOccurrence;
	} Option;
	
	typedef struct {
			StString group, desc, add;
	} Documentation;
	
	public:
		StArgs(){ init(); }
		StArgs( int argc, char* argv[] ){ init(); setArgs( argc, argv ); }
		
		/**  */
		void setArgs( int argc, char* argv[] );
		
		/*!
			Adds option to validate list. If option with given name already exists
			it valueNum and allowMultipleValues will be updated.
			This function also can maintain options documentation for you. For
			further information check out \ref how_to_document_the_arguments.
			\sa linkOption()
			\param name Name of option like -no-deps
			\param valueNum Do option have value? One or more? 
			\param allowMultipleOccurrence If true option can be used multiple times
			\param docGroup defines in which documentation group this option is
			\param optionsAdd is what you want to put after options listing
			\param docDesc description of this option
		*/
		void addOption( 
			       const StString& name,
			       SmartTools::OptionValues valueNum=SmartTools::None,
			       bool allowMultipleOccurrence=false,
			       const StString& docGroup="",
			       const StString& optionsAdd="",
			       const StString& docDesc=""
			       );
		
		/*!
			Make link to existing option if exists. Link can be done to another
			link.
			\return False if linkTo doesn't exists.
			\sa addOption()
		*/
		bool linkOption( const StString& linkName, const StString& linkTo );
		
		/*!
			Disallow presence of more than one option from options at time.
			\return false if one or more options not recognized.
		*/
		bool disallowTogether( const StStringList options );
		
		/*!
			Free args are arguments without option in first place
		*/
		void allowFreeArgs( bool opt ){ allowFree = opt; }
		
		void clear(){ args.clear(); options.clear(); nonOptionValues.clear(); disallow.clear(); docs.clear(); init(); }
		
		/*!
			This reads given arguments by setArgs() or StArgs(int,char*) and
			checks if syntax is corresponding to given options.
			
			\return true if parse/validation was succefull, false if argument line is invalid
		*/
		bool parse();
		
		/*!
			\brief obsolete
			
			Use parse() instead of this function.
		*/
		bool validate();
		
		/*!
			\return True if option given in arguments line.
			\sa getValues()
		*/
		bool optionExists( const StString& name )
			{ return ( findOptionInArgs(name) != -1 ) ? true : false; }
			
		/** Generates the options documentation by given data. Read \ref how_to_document_the_arguments . */
		StString getDocumentation( const StStringList& groupOrder, int tabsize=4 );
		
		/*!
			\param optionName Option name that arguments will be returned. If empty
			function will return free arguments
			\return List of values of given option.
			\sa optionExists()
		*/
		StStringList getValues( const StString& optionName="" );
		StString getValue( const StString& optionName );
	
	private:
		StStringList args;
		StStringList nonOptionValues;
		StValueList<Option> options;
		StValueList<StValueList<int> > disallow;
		StValueList<Documentation> docs; // documentation
		bool allowFree;
		bool validated;
		
		void init(){ allowFree = true; validated = false; }
		int findOption( const StString& name );
		int findOptionInArgs( const StString& name );
		
		// return indexes of baned options with given option
		StValueList<int> getDisallowTogetherOptions( const StString& withOption );
};

/*! \class StArgs
	StArgs class lets you set, validate and maintain your own application
	option/argument list. It will check for options existance, correctnes of
	usage and number of arguments.
	
	\section stargs_define Defining options
	
	You can define arguments by addOption() function like:
	
	\code
	StArgs args( argc, argv );
	args.addOption( "--help", None );
	\endcode
	
	First line creates new StArgs object and sets argc and argv options.
	These options are the same, that application takes in int main()
	function. Adding option means to take a good look in what context
	option is used. Here option "--help" is defined with None arguments.
	
	This means that --help doesn't take any additional option within like
	file list or someting else. There are also 'Single' and 'Multiple'
	choices. This is most basic usage of addOption()
	
	\section stargs_validating Validating
	
	When your options are set it's time to validate argument list. To do so
	just call function StArgs::validate() it will returns bool statement if
	argument line is valid or not.
	
	\code
	if( args.validate() )
		stout << "Your options are right!" << endl;
	else
		stout << "Bad argument line! Try again!" << endl;
	\endcode
	
	\section stargs_check Checking of existence
	
	When you're sure that argument line is correct, you probably want to know
	what have been choosen. To check which options are selected and which are
	not, optionExists() will help you.
	
	\code
	if( args.optionExists("--help") )
		stout << "I will help you!" << endl;
	\endcode
	
	\section stargs_link Linking options
	
	Did you know, that most of application options can be written in at least
	two diffrent ways? Let's take --help option for example. Typing --help for
	every time that you need help is iritating. Better will be -h option.
	You can have both options without changing your code:
	
	\code
	args.linkOption( "-h", "--help" ); // now -h is linked to --help
	
	if( args.optionExists( "-h" ) ){ // equals args.optionExists( "--help" )
		...
	}
	\endcode
	
	You don't need to change your code at all. You can use --help or -h
	string to identify your option. You can also have more than two
	representations of one option. There is no limit!
	
	\section stargs_advenced Advenced options
	
	For more advenced configurations you should take a look to
	allowFreeArgs() (float arguments), disallowTogether() ( banished combinations ),
	getDocumentation() ( automatic --help output generator ), getValues() and getValue().
*/

/*! \page how_to_document_the_arguments Documenting the arguments
	StArgs class provides automatic documentation of your arguments.
	
	\section stargs_doc_a How to documentate your arguments
	
	It's really simple. All you have to do is to add 3-4 more arguments
	to StArgs::addOption() function, which adds your argument to check
	list. Usually it looks like:
	
	\code
	StArgs args( argc, argv );
	args.addOption( "--help", None );
	\endcode
	
	This will add "--help" argument to verify list as far as you should know.
	
	To document this option you must add few more arguments to this function like:
	
	\code
	args.addOption(
		       "--install", // 1
		       Multiple, // 2
		       false, // 3
		       "Operations", // 4
		       "[package list]" // 5
		       "Install selected packages" // 6
		       );
	\endcode
	
	Ok let's look at this. First I chose --install name for option (<b>1</b>).
	Then I set multiple arguments for this option (<b>2</b>). <b>Third</b>
	argument is default argument from StArgs::addOption that sets if option
	can be used multiple times in one run. From number <b>4</b> There is first
	documentation entry.
	
	Documentation divides into sections. These sections have names, and each one
	have at least one appurtenant option. If you use new section name for this
	argument then new section will be defined. If you add another option with
	the same section name then it will be added to existing section. Order
	of options listing depends on addOption() orded in code.
	
	<b>Fifth</b> argument mostly is empty but here is used. This is string, that
	will be written after the options listing. Like:
	
	--install [package_list]
	
	And at least last option <b>6</b>. This is description of option - what this
	option exactly do.
	
	Lets say that we link to --install -i option:
	
	\code
	args.linkOption( "-i", "--install" );
	\endcode
	
	Ok, so here is the output:
	\verbatim
	    Operations:
	    	--install, -i [package list]
			Install selected packages
	\endverbatim
	
*/

#endif //SMART_TOOLS_ARGS_H
